import greenfoot.*;
import java.awt.Color;

/**
 * Append a peice of Matrix code onto the background
 * 
 * @author Jamie Vaughan
 * @version 1.0
 */
public class Code extends Actor {

    /** Store the colour used of this code **/
    public Color colour;
    
    /** Store the speed of which the code falls **/
    public int SPEED = 3;
    
    /** Size of the code **/
    public int SIZE;
    
    /** Store the amount of codes created by this code **/
    public int MODIFIIER = 1;
    
    /** If this code is the leader of others that are created **/
    public boolean IS_LEADER = false;
    
    /** If the code is currently valid or not **/
    public boolean VALID = true;
    
    /** Store the characters string code **/
    public int raw;
    
    /** Store when the last action was held **/
    public long LAST_ACTION = System.currentTimeMillis();

    /**
     * Append a new peice of coding to the world
     * @param colour The colour used for the code
     * @param IS_LEADER if the code is the leader of others
     */
    public Code(Color colour, boolean IS_LEADER, int size) {
        this.colour = colour;
        this.IS_LEADER = IS_LEADER;
        this.raw = '0' + (int)(Math.random() * 2);
        setImage(new GreenfootImage(10, 10));
        SIZE = size;
    }
    
    /**
     * Act - do whatever the Code wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() {
        fall();
        draw();
    }
    
    /**
     * Used to draw the code onto the image
     */
    public void draw() {
        
        /**
         * Don't attempt to repaint if the object is 'invalid'
         */
        if(VALID) {
            GreenfootImage image = new GreenfootImage(15, 15);
            image.setColor(getColour());
            image.drawString(Character.toString((char) raw), 5, getImage().getHeight());
            setImage(image);
            image.scale(SIZE, SIZE);
            duplicate();
        }
    }
    
    /**
     * Creates a follower onto this leading code
     */
    public void duplicate() {
        if(IS_LEADER && System.currentTimeMillis() - LAST_ACTION > Matrix.TIME / 2) {
            getWorld().addObject(new Code(Matrix.CODE_COLOUR, false, SIZE), getX(), getY() - (MODIFIIER++ * 20));
            LAST_ACTION = System.currentTimeMillis();
        }
    }
    
    /**
     * Used to get the colour of the text depending on
     * the distance from the start to the end of the journey
     * @return Returns the colour of the falling code
     */
    public Color getColour() {
        int i = ((getY() / 10) * 7);
        int alpha = i > 255 ? 255 : i;
        return new Color(colour.getRed(), colour.getGreen(), colour.getBlue(), IS_LEADER ? 200 : alpha);
    }
    
    /**
     * Remove this object from the worlds list
     */
    public void remove() {
        VALID = false;
        getWorld().removeObject(this);
    }
    
    /**
     * Makes the code fall down until it reaches the edge
     */
    public void fall() {
        
        /**
         * Once the code reaches the maximum height it will be removed
         */
        if(getY() >= getWorld().getHeight() - 10) {
            remove();
        }
        else {
            
            /**
             * If the code is in contact with another it will be removed
             */
            if(getOneObjectAtOffset(0, getImage().getHeight(), Code.class) == null)
                setLocation(getX(), getY() + SIZE/2);
            else {
                remove();
            }
        }
    }
}
